#--------------------------------------------------------------------------
#
#   ALHM - Player
#
#--------------------------------------------------------------------------

from albow.resource import get_image, get_font, get_sound
from albow.image_array import get_image_array

from constants import *
from functions import *

class Player(object):

    body = get_image("princess.png")
    body_dead = get_image("princess-defeated.png")
    eyes = get_image_array("princess-eyes.png", 4)
    health_indicator = get_image_array("princess-health.png", 13)
    xoffset = 30
    yoffset = 60
    health_xoffset = 5
    health_yoffset = 8
    max_jiggle = 2
    jiggle_rate = 50
    
    #start_cc = (5, 2)
    speed = 150
    
    def __init__(self, level, pos):
        self.position = list(pos) #list(level.cell_centre(self.start_cc))
        self.eye_direction = RIGHT
        self.moving_direction = None
        self.health = 12
        self.jiggle = 0
        self.jiggle_direction = 1

    def draw(self, surf):
        px, py = self.position
        md = self.moving_direction
        if md is not None:
            if md & 1:
                px += self.jiggle
            else:
                py += self.jiggle
        x = px - self.xoffset
        y = py - self.yoffset
        hx = self.health_xoffset
        hy = self.health_yoffset
        h = self.health
        if h > 0:
            surf.blit(self.body, (x, y))
            surf.blit(self.eyes[self.eye_direction], (x + 20, y + 38))
            surf.blit(self.health_indicator[12 - h], (x + hx, y + hy))
        else:
            surf.blit(self.body_dead, (px - self.yoffset, py - self.xoffset))
    
    def start_moving(self, dir):
        self.moving_direction = dir
        #self.eye_direction = dir ^ 2
    
    def stop_moving(self, dir):
        if self.moving_direction == dir:
            self.moving_direction = None
    
    def begin_frame(self, level, dt):
        self.update_jiggle(dt)
        self.update_position(level, dt)
        self.check_traps(level)
        self.update_eyes(level)
    
    def update_jiggle(self, dt):
        if self.moving_direction is not None:
            if self.jiggle_direction > 0:
                self.jiggle += dt * self.jiggle_rate
                if self.jiggle > self.max_jiggle:
                    self.jiggle_direction = -1
            else:
                self.jiggle -= dt * self.jiggle_rate
                if self.jiggle < -self.max_jiggle:
                    self.jiggle_direction = 1
        #print self.jiggle
    
    def update_position(self, level, dt):
        dir = self.moving_direction
        if dir is not None:
            pos = self.position
            cc = level.cc_containing(pos)
            cell = level.get_cell(cc)
            if cell:
                pos_ahead = add(pos, scale(50, dirvec[dir]))
                cc_ahead = level.cc_containing(pos_ahead)
                if cc_ahead == cc or cell.connected[dir]:
                    self.position = add(pos, scale(dt * self.speed, dirvec[dir]))
    
    def check_traps(self, level):
        p = self.position
        for trap in level.traps:
            if trap.bounds.collidepoint(p):
                trap.trample(level)
    
    def sting(self, level):
        self.health -= 1
        if self.health == 0:
            level.game_outcome = -1

    def activation_rect(self):
        r = self.body.get_rect()
        r.center = self.position
        return r

    def update_eyes(self, level):
        dx, dy = sub(level.swarm.centre, self.position)
        if abs(dx) >= abs(dy):
            if dx < 0:
                d = LEFT
            else:
                d = RIGHT
        else:
            if dy < 0:
                d = UP
            else:
                d = DOWN
        self.eye_direction = d
